from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardOrganizationsOauthScopesListOutputPermissions:
    identifier: str
    name: str
    description: str
    dependencies: List[str]
@dataclass
class DashboardOrganizationsOauthScopesListOutput:
    object: str
    permissions: List[DashboardOrganizationsOauthScopesListOutputPermissions]


class mapDashboardOrganizationsOauthScopesListOutputPermissions:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthScopesListOutputPermissions:
        return DashboardOrganizationsOauthScopesListOutputPermissions(
        identifier=data.get('identifier'),
        name=data.get('name'),
        description=data.get('description'),
        dependencies=data.get('dependencies', [])
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthScopesListOutputPermissions, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthScopesListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthScopesListOutput:
        return DashboardOrganizationsOauthScopesListOutput(
        object=data.get('object'),
        permissions=[mapDashboardOrganizationsOauthScopesListOutputPermissions.from_dict(item) for item in data.get('permissions', []) if item]
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthScopesListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftContainerImage:
    object: str
    container_registry: str
    container_image_tag: str
    container_image: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftRemoteMcpServer:
    object: str
    url: str
    transport: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftConfigSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftConfig:
    object: str
    schema: DashboardInstanceCustomProvidersCreateOutputDraftConfigSchema
    transformer: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraft:
    object: str
    config: DashboardInstanceCustomProvidersCreateOutputDraftConfig
    container_image: Optional[DashboardInstanceCustomProvidersCreateOutputDraftContainerImage] = None
    remote_mcp_server: Optional[DashboardInstanceCustomProvidersCreateOutputDraftRemoteMcpServer] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputScmRepoProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputScmRepo:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersCreateOutputScmRepoProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersCreateOutputProviderPublisher:
    object: str
    id: str
    name: str
    image_url: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputProviderCurrentVersion:
    object: str
    id: str
    version: str
    provider_id: str
    is_current: bool
    name: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    specification_id: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputProviderOauthAutoRegistration:
    status: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputProviderOauth:
    status: str
    auto_registration: DashboardInstanceCustomProvidersCreateOutputProviderOauthAutoRegistration
    callback_url: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputProvider:
    object: str
    id: str
    access: str
    status: str
    publisher: DashboardInstanceCustomProvidersCreateOutputProviderPublisher
    identifier: str
    name: str
    slug: str
    created_at: datetime
    updated_at: datetime
    current_version: Optional[DashboardInstanceCustomProvidersCreateOutputProviderCurrentVersion] = None
    oauth: Optional[DashboardInstanceCustomProvidersCreateOutputProviderOauth] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLinkRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLinkRepository:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLinkRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLink:
    object: str
    is_linked: str
    repository: DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLinkRepository
    path: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutputDraftBucket:
    object: str
    id: str
    is_immutable: bool
    is_read_only: bool
    created_at: datetime
    scm_repo_link: Optional[DashboardInstanceCustomProvidersCreateOutputDraftBucketScmRepoLink] = None
@dataclass
class DashboardInstanceCustomProvidersCreateOutput:
    object: str
    id: str
    status: str
    type: str
    name: str
    draft: DashboardInstanceCustomProvidersCreateOutputDraft
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    scm_repo: Optional[DashboardInstanceCustomProvidersCreateOutputScmRepo] = None
    provider: Optional[DashboardInstanceCustomProvidersCreateOutputProvider] = None
    draft_bucket: Optional[DashboardInstanceCustomProvidersCreateOutputDraftBucket] = None


class mapDashboardInstanceCustomProvidersCreateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersCreateOutput:
        return DashboardInstanceCustomProvidersCreateOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        type=data.get('type'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        draft=mapDashboardInstanceCustomProvidersCreateOutputDraft.from_dict(data.get('draft')) if data.get('draft') else None,
        scm_repo=mapDashboardInstanceCustomProvidersCreateOutputScmRepo.from_dict(data.get('scm_repo')) if data.get('scm_repo') else None,
        provider=mapDashboardInstanceCustomProvidersCreateOutputProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None,
        draft_bucket=mapDashboardInstanceCustomProvidersCreateOutputDraftBucket.from_dict(data.get('draft_bucket')) if data.get('draft_bucket') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersCreateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardInstanceCustomProvidersCreateBodyConfig:
    schema: Dict[str, Any]
    transformer: str
@dataclass
class DashboardInstanceCustomProvidersCreateBody:
    name: str
    from_: Dict[str, Any]
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    config: Optional[DashboardInstanceCustomProvidersCreateBodyConfig] = None


class mapDashboardInstanceCustomProvidersCreateBodyConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersCreateBodyConfig:
        return DashboardInstanceCustomProvidersCreateBodyConfig(
        schema=data.get('schema'),
        transformer=data.get('transformer')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersCreateBodyConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersCreateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersCreateBody:
        return DashboardInstanceCustomProvidersCreateBody(
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        from_=data.get('from'),
        config=mapDashboardInstanceCustomProvidersCreateBodyConfig.from_dict(data.get('config')) if data.get('config') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersCreateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsMcpClient:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsMcpServer:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsMcp:
  object: str
  version: str
  connection_type: str
  client: Optional[
    ManagementInstanceSessionsServerSessionsListOutputItemsMcpClient
  ] = None
  server: Optional[
    ManagementInstanceSessionsServerSessionsListOutputItemsMcpServer
  ] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsSession:
  object: str
  id: str
  status: str
  connection_status: str
  usage: ManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: ManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient:
  user_agent: str
  anonymized_ip_address: str


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItemsConnection:
  object: str
  id: str
  client: ManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient
  created_at: datetime
  started_at: datetime
  ended_at: Optional[datetime] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputItems:
  object: str
  id: str
  status: str
  mcp: ManagementInstanceSessionsServerSessionsListOutputItemsMcp
  usage: ManagementInstanceSessionsServerSessionsListOutputItemsUsage
  server: ManagementInstanceSessionsServerSessionsListOutputItemsServer
  session: ManagementInstanceSessionsServerSessionsListOutputItemsSession
  server_deployment: ManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment
  created_at: datetime
  connection: Optional[
    ManagementInstanceSessionsServerSessionsListOutputItemsConnection
  ] = None


@dataclass
class ManagementInstanceSessionsServerSessionsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class ManagementInstanceSessionsServerSessionsListOutput:
  items: List[ManagementInstanceSessionsServerSessionsListOutputItems]
  pagination: ManagementInstanceSessionsServerSessionsListOutputPagination


class mapManagementInstanceSessionsServerSessionsListOutputItemsMcpClient:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsMcpClient:
    return ManagementInstanceSessionsServerSessionsListOutputItemsMcpClient(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsMcpClient,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsMcpServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsMcpServer:
    return ManagementInstanceSessionsServerSessionsListOutputItemsMcpServer(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsMcpServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsMcp:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsMcp:
    return ManagementInstanceSessionsServerSessionsListOutputItemsMcp(
      object=data.get("object"),
      version=data.get("version"),
      connection_type=data.get("connection_type"),
      client=mapManagementInstanceSessionsServerSessionsListOutputItemsMcpClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      server=mapManagementInstanceSessionsServerSessionsListOutputItemsMcpServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsMcp, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsUsage:
    return ManagementInstanceSessionsServerSessionsListOutputItemsUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsUsage, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsServer:
    return ManagementInstanceSessionsServerSessionsListOutputItemsServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage:
    return ManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsSession:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsSession:
    return ManagementInstanceSessionsServerSessionsListOutputItemsSession(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      connection_status=data.get("connection_status"),
      usage=mapManagementInstanceSessionsServerSessionsListOutputItemsSessionUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsSession,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer:
    return (
      ManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer(
        object=data.get("object"),
        id=data.get("id"),
        name=data.get("name"),
        description=data.get("description"),
        type=data.get("type"),
        created_at=parse_iso_datetime(data.get("created_at"))
        if data.get("created_at")
        else None,
        updated_at=parse_iso_datetime(data.get("updated_at"))
        if data.get("updated_at")
        else None,
      )
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment:
    return ManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapManagementInstanceSessionsServerSessionsListOutputItemsServerDeploymentServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient:
    return ManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient(
      user_agent=data.get("user_agent"),
      anonymized_ip_address=data.get("anonymized_ip_address"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItemsConnection:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItemsConnection:
    return ManagementInstanceSessionsServerSessionsListOutputItemsConnection(
      object=data.get("object"),
      id=data.get("id"),
      client=mapManagementInstanceSessionsServerSessionsListOutputItemsConnectionClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      started_at=parse_iso_datetime(data.get("started_at"))
      if data.get("started_at")
      else None,
      ended_at=parse_iso_datetime(data.get("ended_at"))
      if data.get("ended_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItemsConnection,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputItems:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputItems:
    return ManagementInstanceSessionsServerSessionsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      mcp=mapManagementInstanceSessionsServerSessionsListOutputItemsMcp.from_dict(
        data.get("mcp")
      )
      if data.get("mcp")
      else None,
      usage=mapManagementInstanceSessionsServerSessionsListOutputItemsUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      server=mapManagementInstanceSessionsServerSessionsListOutputItemsServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      session=mapManagementInstanceSessionsServerSessionsListOutputItemsSession.from_dict(
        data.get("session")
      )
      if data.get("session")
      else None,
      server_deployment=mapManagementInstanceSessionsServerSessionsListOutputItemsServerDeployment.from_dict(
        data.get("server_deployment")
      )
      if data.get("server_deployment")
      else None,
      connection=mapManagementInstanceSessionsServerSessionsListOutputItemsConnection.from_dict(
        data.get("connection")
      )
      if data.get("connection")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputItems, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutputPagination:
    return ManagementInstanceSessionsServerSessionsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutputPagination, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceSessionsServerSessionsListOutput:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListOutput:
    return ManagementInstanceSessionsServerSessionsListOutput(
      items=[
        mapManagementInstanceSessionsServerSessionsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapManagementInstanceSessionsServerSessionsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListOutput, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class ManagementInstanceSessionsServerSessionsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None


class mapManagementInstanceSessionsServerSessionsListQuery:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceSessionsServerSessionsListQuery:
    return ManagementInstanceSessionsServerSessionsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceSessionsServerSessionsListQuery, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes:
    identifier: str
    name: str
    description: str
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes:
    identifier: str
    name: str
    description: str
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets:
    object: str
    id: str
    preview: str
    created_at: datetime
    secret: Optional[str] = None
    deleted_at: Optional[datetime] = None
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication:
    object: str
    id: str
    status: str
    type: str
    access_level: str
    allow_token_exchange_without_client_secret: bool
    name: str
    scopes: List[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes]
    image_url: str
    redirect_uris: List[str]
    client_id: str
    client_secrets: List[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets]
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    website_url: Optional[str] = None
    privacy_policy_url: Optional[str] = None
    terms_of_service_url: Optional[str] = None
    organization_id: Optional[str] = None
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams:
    id: str
    name: str
    slug: str
    assignment_id: str
    created_at: datetime
    updated_at: datetime
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor:
    object: str
    id: str
    type: str
    organization_id: str
    name: str
    image_url: str
    teams: List[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams]
    created_at: datetime
    updated_at: datetime
    email: Optional[str] = None
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser:
    object: str
    id: str
    status: str
    type: str
    email: str
    name: str
    first_name: str
    last_name: str
    image_url: str
    created_at: datetime
    updated_at: datetime
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputItems:
    object: str
    id: str
    status: str
    type: str
    organization_id: str
    scopes: List[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes]
    oauth_application: DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication
    created_at: datetime
    redirect_uri: Optional[str] = None
    client_ip: Optional[str] = None
    actor: Optional[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor] = None
    user: Optional[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser] = None
    accepted_at: Optional[datetime] = None
    denied_at: Optional[datetime] = None
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListOutput:
    items: List[DashboardOrganizationsOauthAuthorizationLogsListOutputItems]
    pagination: DashboardOrganizationsOauthAuthorizationLogsListOutputPagination


class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes(
        identifier=data.get('identifier'),
        name=data.get('name'),
        description=data.get('description')
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes(
        identifier=data.get('identifier'),
        name=data.get('name'),
        description=data.get('description')
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets(
        object=data.get('object'),
        id=data.get('id'),
        preview=data.get('preview'),
        secret=data.get('secret'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        deleted_at=datetime.fromisoformat(data.get('deleted_at').replace('Z', '+00:00')) if data.get('deleted_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        type=data.get('type'),
        access_level=data.get('access_level'),
        allow_token_exchange_without_client_secret=data.get('allow_token_exchange_without_client_secret'),
        name=data.get('name'),
        description=data.get('description'),
        scopes=[mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationScopes.from_dict(item) for item in data.get('scopes', []) if item],
        image_url=data.get('image_url'),
        website_url=data.get('website_url'),
        privacy_policy_url=data.get('privacy_policy_url'),
        terms_of_service_url=data.get('terms_of_service_url'),
        redirect_uris=data.get('redirect_uris', []),
        client_id=data.get('client_id'),
        client_secrets=[mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplicationClientSecrets.from_dict(item) for item in data.get('client_secrets', []) if item],
        organization_id=data.get('organization_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams(
        id=data.get('id'),
        name=data.get('name'),
        slug=data.get('slug'),
        assignment_id=data.get('assignment_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        organization_id=data.get('organization_id'),
        name=data.get('name'),
        email=data.get('email'),
        image_url=data.get('image_url'),
        teams=[mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsActorTeams.from_dict(item) for item in data.get('teams', []) if item],
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        type=data.get('type'),
        email=data.get('email'),
        name=data.get('name'),
        first_name=data.get('first_name'),
        last_name=data.get('last_name'),
        image_url=data.get('image_url'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputItems:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        type=data.get('type'),
        organization_id=data.get('organization_id'),
        redirect_uri=data.get('redirect_uri'),
        client_ip=data.get('client_ip'),
        scopes=[mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsScopes.from_dict(item) for item in data.get('scopes', []) if item],
        oauth_application=mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsOauthApplication.from_dict(data.get('oauth_application')) if data.get('oauth_application') else None,
        actor=mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsActor.from_dict(data.get('actor')) if data.get('actor') else None,
        user=mapDashboardOrganizationsOauthAuthorizationLogsListOutputItemsUser.from_dict(data.get('user')) if data.get('user') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        accepted_at=datetime.fromisoformat(data.get('accepted_at').replace('Z', '+00:00')) if data.get('accepted_at') else None,
        denied_at=datetime.fromisoformat(data.get('denied_at').replace('Z', '+00:00')) if data.get('denied_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutputPagination:
        return DashboardOrganizationsOauthAuthorizationLogsListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardOrganizationsOauthAuthorizationLogsListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListOutput:
        return DashboardOrganizationsOauthAuthorizationLogsListOutput(
        items=[mapDashboardOrganizationsOauthAuthorizationLogsListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapDashboardOrganizationsOauthAuthorizationLogsListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardOrganizationsOauthAuthorizationLogsListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    app_id: Optional[Union[str, List[str]]] = None
    user_id: Optional[Union[str, List[str]]] = None


class mapDashboardOrganizationsOauthAuthorizationLogsListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardOrganizationsOauthAuthorizationLogsListQuery:
        return DashboardOrganizationsOauthAuthorizationLogsListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        app_id=data.get('app_id'),
        user_id=data.get('user_id')
        )

    @staticmethod
    def to_dict(value: Union[DashboardOrganizationsOauthAuthorizationLogsListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ServerRunsGetOutputServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ServerRunsGetOutputServerDeploymentServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ServerRunsGetOutputServerDeployment:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: ServerRunsGetOutputServerDeploymentServer
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class ServerRunsGetOutputServerSessionMcpClient:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ServerRunsGetOutputServerSessionMcpServer:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ServerRunsGetOutputServerSessionMcp:
  object: str
  version: str
  connection_type: str
  client: Optional[ServerRunsGetOutputServerSessionMcpClient] = None
  server: Optional[ServerRunsGetOutputServerSessionMcpServer] = None


@dataclass
class ServerRunsGetOutputServerSessionUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class ServerRunsGetOutputServerSession:
  object: str
  id: str
  status: str
  mcp: ServerRunsGetOutputServerSessionMcp
  usage: ServerRunsGetOutputServerSessionUsage
  session_id: str
  created_at: datetime


@dataclass
class ServerRunsGetOutput:
  object: str
  id: str
  type: str
  status: str
  server_version_id: str
  server: ServerRunsGetOutputServer
  server_deployment: ServerRunsGetOutputServerDeployment
  server_session: ServerRunsGetOutputServerSession
  created_at: datetime
  updated_at: datetime
  started_at: Optional[datetime] = None
  stopped_at: Optional[datetime] = None


class mapServerRunsGetOutputServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServer:
    return ServerRunsGetOutputServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerDeploymentServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerDeploymentServer:
    return ServerRunsGetOutputServerDeploymentServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerDeploymentServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerDeployment:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerDeployment:
    return ServerRunsGetOutputServerDeployment(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapServerRunsGetOutputServerDeploymentServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerDeployment, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerSessionMcpClient:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerSessionMcpClient:
    return ServerRunsGetOutputServerSessionMcpClient(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerSessionMcpClient, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerSessionMcpServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerSessionMcpServer:
    return ServerRunsGetOutputServerSessionMcpServer(
      object=data.get("object"),
      name=data.get("name"),
      version=data.get("version"),
      capabilities=data.get("capabilities"),
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerSessionMcpServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerSessionMcp:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerSessionMcp:
    return ServerRunsGetOutputServerSessionMcp(
      object=data.get("object"),
      version=data.get("version"),
      connection_type=data.get("connection_type"),
      client=mapServerRunsGetOutputServerSessionMcpClient.from_dict(data.get("client"))
      if data.get("client")
      else None,
      server=mapServerRunsGetOutputServerSessionMcpServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerSessionMcp, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerSessionUsage:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerSessionUsage:
    return ServerRunsGetOutputServerSessionUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerSessionUsage, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutputServerSession:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutputServerSession:
    return ServerRunsGetOutputServerSession(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      mcp=mapServerRunsGetOutputServerSessionMcp.from_dict(data.get("mcp"))
      if data.get("mcp")
      else None,
      usage=mapServerRunsGetOutputServerSessionUsage.from_dict(data.get("usage"))
      if data.get("usage")
      else None,
      session_id=data.get("session_id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutputServerSession, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServerRunsGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServerRunsGetOutput:
    return ServerRunsGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      type=data.get("type"),
      status=data.get("status"),
      server_version_id=data.get("server_version_id"),
      server=mapServerRunsGetOutputServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
      server_deployment=mapServerRunsGetOutputServerDeployment.from_dict(
        data.get("server_deployment")
      )
      if data.get("server_deployment")
      else None,
      server_session=mapServerRunsGetOutputServerSession.from_dict(
        data.get("server_session")
      )
      if data.get("server_session")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      started_at=parse_iso_datetime(data.get("started_at"))
      if data.get("started_at")
      else None,
      stopped_at=parse_iso_datetime(data.get("stopped_at"))
      if data.get("stopped_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServerRunsGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from .resources import *
from .endpoints import *

# Type exports for better discoverability
# Generated type exports
# These types are automatically exported for better IDE support

# Type classes
# ApiKeysCreateBody
# ApiKeysCreateOutput
# ApiKeysGetOutput
# ApiKeysListOutput
# ApiKeysListQuery
# ApiKeysRevealOutput
# ApiKeysRevokeOutput
# ApiKeysRotateBody
# ApiKeysRotateOutput
# ApiKeysUpdateBody
# ApiKeysUpdateOutput
# CustomServersManagedServerTemplatesGetOutput
# CustomServersManagedServerTemplatesListOutput
# CustomServersManagedServerTemplatesListQuery
# DashboardBootBody
# DashboardBootOutput
# DashboardInstanceCallbacksDestinationsCreateBody
# DashboardInstanceCallbacksDestinationsCreateOutput
# DashboardInstanceCallbacksDestinationsDeleteOutput
# DashboardInstanceCallbacksDestinationsGetOutput
# DashboardInstanceCallbacksDestinationsListOutput
# DashboardInstanceCallbacksDestinationsListQuery
# DashboardInstanceCallbacksDestinationsUpdateBody
# DashboardInstanceCallbacksDestinationsUpdateOutput
# DashboardInstanceCallbacksEventsGetOutput
# DashboardInstanceCallbacksEventsListOutput
# DashboardInstanceCallbacksEventsListQuery
# DashboardInstanceCallbacksGetOutput
# DashboardInstanceCallbacksListOutput
# DashboardInstanceCallbacksListQuery
# DashboardInstanceCallbacksNotificationsGetOutput
# DashboardInstanceCallbacksNotificationsListOutput
# DashboardInstanceCallbacksNotificationsListQuery
# DashboardInstanceCustomServersCodeGetCodeEditorTokenOutput
# DashboardInstanceCustomServersCreateBody
# DashboardInstanceCustomServersCreateOutput
# DashboardInstanceCustomServersDeleteOutput
# DashboardInstanceCustomServersDeploymentsGetOutput
# DashboardInstanceCustomServersDeploymentsListOutput
# DashboardInstanceCustomServersDeploymentsListQuery
# DashboardInstanceCustomServersEventsGetOutput
# DashboardInstanceCustomServersEventsListOutput
# DashboardInstanceCustomServersEventsListQuery
# DashboardInstanceCustomServersGetOutput
# DashboardInstanceCustomServersListOutput
# DashboardInstanceCustomServersListQuery
# DashboardInstanceCustomServersListingGetOutput
# DashboardInstanceCustomServersListingUpdateBody
# DashboardInstanceCustomServersListingUpdateOutput
# DashboardInstanceCustomServersRemoteServersGetOutput
# DashboardInstanceCustomServersRemoteServersListOutput
# DashboardInstanceCustomServersRemoteServersListQuery
# DashboardInstanceCustomServersUpdateBody
# DashboardInstanceCustomServersUpdateOutput
# DashboardInstanceCustomServersVersionsCreateBody
# DashboardInstanceCustomServersVersionsCreateOutput
# DashboardInstanceCustomServersVersionsGetOutput
# DashboardInstanceCustomServersVersionsListOutput
# DashboardInstanceCustomServersVersionsListQuery
# DashboardInstanceFilesDeleteOutput
# DashboardInstanceFilesGetOutput
# DashboardInstanceFilesListOutput
# DashboardInstanceFilesListQuery
# DashboardInstanceFilesUpdateBody
# DashboardInstanceFilesUpdateOutput
# DashboardInstanceLinksCreateBody
# DashboardInstanceLinksCreateOutput
# DashboardInstanceLinksDeleteOutput
# DashboardInstanceLinksGetOutput
# DashboardInstanceLinksListOutput
# DashboardInstanceLinksUpdateBody
# DashboardInstanceLinksUpdateOutput
# DashboardInstanceMagicMcpServersCreateBody
# DashboardInstanceMagicMcpServersCreateOutput
# DashboardInstanceMagicMcpServersDeleteOutput
# DashboardInstanceMagicMcpServersGetOutput
# DashboardInstanceMagicMcpServersListOutput
# DashboardInstanceMagicMcpServersListQuery
# DashboardInstanceMagicMcpServersUpdateBody
# DashboardInstanceMagicMcpServersUpdateOutput
# DashboardInstanceMagicMcpSessionsGetOutput
# DashboardInstanceMagicMcpSessionsListOutput
# DashboardInstanceMagicMcpSessionsListQuery
# DashboardInstanceMagicMcpTokensCreateBody
# DashboardInstanceMagicMcpTokensCreateOutput
# DashboardInstanceMagicMcpTokensDeleteOutput
# DashboardInstanceMagicMcpTokensGetOutput
# DashboardInstanceMagicMcpTokensListOutput
# DashboardInstanceMagicMcpTokensListQuery
# DashboardInstanceMagicMcpTokensUpdateBody
# DashboardInstanceMagicMcpTokensUpdateOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsGetOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsListOutput
# DashboardInstanceProviderOauthConnectionsAuthenticationsListQuery
# DashboardInstanceProviderOauthConnectionsCreateBody
# DashboardInstanceProviderOauthConnectionsCreateOutput
# DashboardInstanceProviderOauthConnectionsDeleteOutput
# DashboardInstanceProviderOauthConnectionsEventsGetOutput
# DashboardInstanceProviderOauthConnectionsEventsListOutput
# DashboardInstanceProviderOauthConnectionsEventsListQuery
# DashboardInstanceProviderOauthConnectionsGetOutput
# DashboardInstanceProviderOauthConnectionsListOutput
# DashboardInstanceProviderOauthConnectionsListQuery
# DashboardInstanceProviderOauthConnectionsProfilesGetOutput
# DashboardInstanceProviderOauthConnectionsProfilesListOutput
# DashboardInstanceProviderOauthConnectionsProfilesListQuery
# DashboardInstanceProviderOauthConnectionsUpdateBody
# DashboardInstanceProviderOauthConnectionsUpdateOutput
# DashboardInstanceProviderOauthSessionsCreateBody
# DashboardInstanceProviderOauthSessionsCreateOutput
# DashboardInstanceProviderOauthSessionsDeleteOutput
# DashboardInstanceProviderOauthSessionsGetOutput
# DashboardInstanceProviderOauthSessionsListOutput
# DashboardInstanceProviderOauthSessionsListQuery
# DashboardInstanceProviderOauthTakeoutsCreateBody
# DashboardInstanceProviderOauthTakeoutsCreateOutput
# DashboardInstanceProviderOauthTakeoutsGetOutput
# DashboardInstanceProviderOauthTakeoutsListOutput
# DashboardInstanceProviderOauthTakeoutsListQuery
# DashboardInstanceSecretsGetOutput
# DashboardInstanceSecretsListOutput
# DashboardInstanceSecretsListQuery
# DashboardInstanceServerConfigVaultsCreateBody
# DashboardInstanceServerConfigVaultsCreateOutput
# DashboardInstanceServerConfigVaultsGetOutput
# DashboardInstanceServerConfigVaultsListOutput
# DashboardInstanceServerConfigVaultsListQuery
# DashboardInstanceServerConfigVaultsUpdateBody
# DashboardInstanceServerConfigVaultsUpdateOutput
# DashboardInstanceServerRunErrorGroupsGetOutput
# DashboardInstanceServerRunErrorGroupsListOutput
# DashboardInstanceServerRunErrorGroupsListQuery
# DashboardInstanceServerRunErrorsGetOutput
# DashboardInstanceServerRunErrorsListOutput
# DashboardInstanceServerRunErrorsListQuery
# DashboardInstanceServerRunsGetOutput
# DashboardInstanceServerRunsListOutput
# DashboardInstanceServerRunsListQuery
# DashboardInstanceServersCapabilitiesListOutput
# DashboardInstanceServersCapabilitiesListQuery
# DashboardInstanceServersDeploymentsCreateBody
# DashboardInstanceServersDeploymentsCreateOutput
# DashboardInstanceServersDeploymentsDeleteOutput
# DashboardInstanceServersDeploymentsGetOutput
# DashboardInstanceServersDeploymentsListOutput
# DashboardInstanceServersDeploymentsListQuery
# DashboardInstanceServersDeploymentsUpdateBody
# DashboardInstanceServersDeploymentsUpdateOutput
# DashboardInstanceServersGetOutput
# DashboardInstanceServersImplementationsCreateBody
# DashboardInstanceServersImplementationsCreateOutput
# DashboardInstanceServersImplementationsDeleteOutput
# DashboardInstanceServersImplementationsGetOutput
# DashboardInstanceServersImplementationsListOutput
# DashboardInstanceServersImplementationsListQuery
# DashboardInstanceServersImplementationsUpdateBody
# DashboardInstanceServersImplementationsUpdateOutput
# DashboardInstanceServersVariantsGetOutput
# DashboardInstanceServersVariantsListOutput
# DashboardInstanceServersVariantsListQuery
# DashboardInstanceServersVersionsGetOutput
# DashboardInstanceServersVersionsListOutput
# DashboardInstanceServersVersionsListQuery
# DashboardInstanceSessionsConnectionsGetOutput
# DashboardInstanceSessionsConnectionsListOutput
# DashboardInstanceSessionsConnectionsListQuery
# DashboardInstanceSessionsCreateBody
# DashboardInstanceSessionsCreateOutput
# DashboardInstanceSessionsDeleteOutput
# DashboardInstanceSessionsEventsGetOutput
# DashboardInstanceSessionsEventsListOutput
# DashboardInstanceSessionsEventsListQuery
# DashboardInstanceSessionsGetOutput
# DashboardInstanceSessionsListOutput
# DashboardInstanceSessionsListQuery
# DashboardInstanceSessionsMessagesGetOutput
# DashboardInstanceSessionsMessagesListOutput
# DashboardInstanceSessionsMessagesListQuery
# DashboardInstanceSessionsServerSessionsGetOutput
# DashboardInstanceSessionsServerSessionsListOutput
# DashboardInstanceSessionsServerSessionsListQuery
# DashboardOrganizationsCreateBody
# DashboardOrganizationsCreateOutput
# DashboardOrganizationsDeleteOutput
# DashboardOrganizationsGetOutput
# DashboardOrganizationsInstancesCreateBody
# DashboardOrganizationsInstancesCreateOutput
# DashboardOrganizationsInstancesDeleteOutput
# DashboardOrganizationsInstancesGetOutput
# DashboardOrganizationsInstancesListOutput
# DashboardOrganizationsInstancesListQuery
# DashboardOrganizationsInstancesUpdateBody
# DashboardOrganizationsInstancesUpdateOutput
# DashboardOrganizationsInvitesCreateBody
# DashboardOrganizationsInvitesCreateOutput
# DashboardOrganizationsInvitesDeleteOutput
# DashboardOrganizationsInvitesEnsureLinkOutput
# DashboardOrganizationsInvitesGetOutput
# DashboardOrganizationsInvitesListOutput
# DashboardOrganizationsInvitesListQuery
# DashboardOrganizationsInvitesUpdateBody
# DashboardOrganizationsInvitesUpdateOutput
# DashboardOrganizationsJoinAcceptBody
# DashboardOrganizationsJoinAcceptOutput
# DashboardOrganizationsJoinGetOutput
# DashboardOrganizationsJoinGetQuery
# DashboardOrganizationsJoinRejectBody
# DashboardOrganizationsJoinRejectOutput
# DashboardOrganizationsListOutput
# DashboardOrganizationsListQuery
# DashboardOrganizationsMembersDeleteOutput
# DashboardOrganizationsMembersGetOutput
# DashboardOrganizationsMembersListOutput
# DashboardOrganizationsMembersListQuery
# DashboardOrganizationsMembersUpdateBody
# DashboardOrganizationsMembersUpdateOutput
# DashboardOrganizationsProjectsCreateBody
# DashboardOrganizationsProjectsCreateOutput
# DashboardOrganizationsProjectsDeleteOutput
# DashboardOrganizationsProjectsGetOutput
# DashboardOrganizationsProjectsListOutput
# DashboardOrganizationsProjectsListQuery
# DashboardOrganizationsProjectsUpdateBody
# DashboardOrganizationsProjectsUpdateOutput
# DashboardOrganizationsTeamsCreateBody
# DashboardOrganizationsTeamsCreateOutput
# DashboardOrganizationsTeamsGetOutput
# DashboardOrganizationsTeamsListOutput
# DashboardOrganizationsTeamsListQuery
# DashboardOrganizationsTeamsMembersCreateBody
# DashboardOrganizationsTeamsMembersCreateOutput
# DashboardOrganizationsTeamsMembersDeleteOutput
# DashboardOrganizationsTeamsPermissionsOutput
# DashboardOrganizationsTeamsProjectsRemoveOutput
# DashboardOrganizationsTeamsProjectsSetBody
# DashboardOrganizationsTeamsProjectsSetOutput
# DashboardOrganizationsTeamsRolesCreateBody
# DashboardOrganizationsTeamsRolesCreateOutput
# DashboardOrganizationsTeamsRolesGetOutput
# DashboardOrganizationsTeamsRolesListOutput
# DashboardOrganizationsTeamsRolesListQuery
# DashboardOrganizationsTeamsRolesUpdateBody
# DashboardOrganizationsTeamsRolesUpdateOutput
# DashboardOrganizationsTeamsUpdateBody
# DashboardOrganizationsTeamsUpdateOutput
# DashboardOrganizationsUpdateBody
# DashboardOrganizationsUpdateOutput
# DashboardScmAccountsPreviewOutput
# DashboardScmAccountsPreviewQuery
# DashboardScmInstallationsCreateBody
# DashboardScmInstallationsCreateOutput
# DashboardScmInstallationsGetOutput
# DashboardScmInstallationsListOutput
# DashboardScmInstallationsListQuery
# DashboardScmReposCreateBody
# DashboardScmReposCreateOutput
# DashboardScmReposPreviewOutput
# DashboardScmReposPreviewQuery
# DashboardUsageTimelineOutput
# DashboardUsageTimelineQuery
# ManagementOrganizationGetOutput
# ManagementOrganizationUpdateBody
# ManagementOrganizationUpdateOutput
# ManagementUserDeleteBody
# ManagementUserDeleteOutput
# ManagementUserGetOutput
# ManagementUserUpdateBody
# ManagementUserUpdateOutput
# OrganizationsProfileGetOutput
# OrganizationsProfileUpdateBody
# OrganizationsProfileUpdateOutput
# ProviderOauthConnectionTemplateEvaluateBody
# ProviderOauthConnectionTemplateEvaluateOutput
# ProviderOauthConnectionTemplateGetOutput
# ProviderOauthConnectionTemplateListOutput
# ProviderOauthConnectionTemplateListQuery
# ProviderOauthDiscoverBody
# ProviderOauthDiscoverOutput
# ServersListingsCategoriesGetOutput
# ServersListingsCategoriesListOutput
# ServersListingsCategoriesListQuery
# ServersListingsCollectionsGetOutput
# ServersListingsCollectionsListOutput
# ServersListingsCollectionsListQuery
# ServersListingsGetOutput
# ServersListingsGetQuery
# ServersListingsListOutput
# ServersListingsListQuery

# Mapper classes
# mapApiKeysCreateBody
# mapApiKeysCreateOutput
# mapApiKeysGetOutput
# mapApiKeysListOutput
# mapApiKeysListQuery
# mapApiKeysRevealOutput
# mapApiKeysRevokeOutput
# mapApiKeysRotateBody
# mapApiKeysRotateOutput
# mapApiKeysUpdateBody
# mapApiKeysUpdateOutput
# mapCustomServersManagedServerTemplatesGetOutput
# mapCustomServersManagedServerTemplatesListOutput
# mapCustomServersManagedServerTemplatesListQuery
# mapDashboardBootBody
# mapDashboardBootOutput
# mapDashboardInstanceCallbacksDestinationsCreateBody
# mapDashboardInstanceCallbacksDestinationsCreateOutput
# mapDashboardInstanceCallbacksDestinationsDeleteOutput
# mapDashboardInstanceCallbacksDestinationsGetOutput
# mapDashboardInstanceCallbacksDestinationsListOutput
# mapDashboardInstanceCallbacksDestinationsListQuery
# mapDashboardInstanceCallbacksDestinationsUpdateBody
# mapDashboardInstanceCallbacksDestinationsUpdateOutput
# mapDashboardInstanceCallbacksEventsGetOutput
# mapDashboardInstanceCallbacksEventsListOutput
# mapDashboardInstanceCallbacksEventsListQuery
# mapDashboardInstanceCallbacksGetOutput
# mapDashboardInstanceCallbacksListOutput
# mapDashboardInstanceCallbacksListQuery
# mapDashboardInstanceCallbacksNotificationsGetOutput
# mapDashboardInstanceCallbacksNotificationsListOutput
# mapDashboardInstanceCallbacksNotificationsListQuery
# mapDashboardInstanceCustomServersCodeGetCodeEditorTokenOutput
# mapDashboardInstanceCustomServersCreateBody
# mapDashboardInstanceCustomServersCreateOutput
# mapDashboardInstanceCustomServersDeleteOutput
# mapDashboardInstanceCustomServersDeploymentsGetOutput
# mapDashboardInstanceCustomServersDeploymentsListOutput
# mapDashboardInstanceCustomServersDeploymentsListQuery
# mapDashboardInstanceCustomServersEventsGetOutput
# mapDashboardInstanceCustomServersEventsListOutput
# mapDashboardInstanceCustomServersEventsListQuery
# mapDashboardInstanceCustomServersGetOutput
# mapDashboardInstanceCustomServersListOutput
# mapDashboardInstanceCustomServersListQuery
# mapDashboardInstanceCustomServersListingGetOutput
# mapDashboardInstanceCustomServersListingUpdateBody
# mapDashboardInstanceCustomServersListingUpdateOutput
# mapDashboardInstanceCustomServersRemoteServersGetOutput
# mapDashboardInstanceCustomServersRemoteServersListOutput
# mapDashboardInstanceCustomServersRemoteServersListQuery
# mapDashboardInstanceCustomServersUpdateBody
# mapDashboardInstanceCustomServersUpdateOutput
# mapDashboardInstanceCustomServersVersionsCreateBody
# mapDashboardInstanceCustomServersVersionsCreateOutput
# mapDashboardInstanceCustomServersVersionsGetOutput
# mapDashboardInstanceCustomServersVersionsListOutput
# mapDashboardInstanceCustomServersVersionsListQuery
# mapDashboardInstanceFilesDeleteOutput
# mapDashboardInstanceFilesGetOutput
# mapDashboardInstanceFilesListOutput
# mapDashboardInstanceFilesListQuery
# mapDashboardInstanceFilesUpdateBody
# mapDashboardInstanceFilesUpdateOutput
# mapDashboardInstanceLinksCreateBody
# mapDashboardInstanceLinksCreateOutput
# mapDashboardInstanceLinksDeleteOutput
# mapDashboardInstanceLinksGetOutput
# mapDashboardInstanceLinksListOutput
# mapDashboardInstanceLinksUpdateBody
# mapDashboardInstanceLinksUpdateOutput
# mapDashboardInstanceMagicMcpServersCreateBody
# mapDashboardInstanceMagicMcpServersCreateOutput
# mapDashboardInstanceMagicMcpServersDeleteOutput
# mapDashboardInstanceMagicMcpServersGetOutput
# mapDashboardInstanceMagicMcpServersListOutput
# mapDashboardInstanceMagicMcpServersListQuery
# mapDashboardInstanceMagicMcpServersUpdateBody
# mapDashboardInstanceMagicMcpServersUpdateOutput
# mapDashboardInstanceMagicMcpSessionsGetOutput
# mapDashboardInstanceMagicMcpSessionsListOutput
# mapDashboardInstanceMagicMcpSessionsListQuery
# mapDashboardInstanceMagicMcpTokensCreateBody
# mapDashboardInstanceMagicMcpTokensCreateOutput
# mapDashboardInstanceMagicMcpTokensDeleteOutput
# mapDashboardInstanceMagicMcpTokensGetOutput
# mapDashboardInstanceMagicMcpTokensListOutput
# mapDashboardInstanceMagicMcpTokensListQuery
# mapDashboardInstanceMagicMcpTokensUpdateBody
# mapDashboardInstanceMagicMcpTokensUpdateOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsGetOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsListOutput
# mapDashboardInstanceProviderOauthConnectionsAuthenticationsListQuery
# mapDashboardInstanceProviderOauthConnectionsCreateBody
# mapDashboardInstanceProviderOauthConnectionsCreateOutput
# mapDashboardInstanceProviderOauthConnectionsDeleteOutput
# mapDashboardInstanceProviderOauthConnectionsEventsGetOutput
# mapDashboardInstanceProviderOauthConnectionsEventsListOutput
# mapDashboardInstanceProviderOauthConnectionsEventsListQuery
# mapDashboardInstanceProviderOauthConnectionsGetOutput
# mapDashboardInstanceProviderOauthConnectionsListOutput
# mapDashboardInstanceProviderOauthConnectionsListQuery
# mapDashboardInstanceProviderOauthConnectionsProfilesGetOutput
# mapDashboardInstanceProviderOauthConnectionsProfilesListOutput
# mapDashboardInstanceProviderOauthConnectionsProfilesListQuery
# mapDashboardInstanceProviderOauthConnectionsUpdateBody
# mapDashboardInstanceProviderOauthConnectionsUpdateOutput
# mapDashboardInstanceProviderOauthSessionsCreateBody
# mapDashboardInstanceProviderOauthSessionsCreateOutput
# mapDashboardInstanceProviderOauthSessionsDeleteOutput
# mapDashboardInstanceProviderOauthSessionsGetOutput
# mapDashboardInstanceProviderOauthSessionsListOutput
# mapDashboardInstanceProviderOauthSessionsListQuery
# mapDashboardInstanceProviderOauthTakeoutsCreateBody
# mapDashboardInstanceProviderOauthTakeoutsCreateOutput
# mapDashboardInstanceProviderOauthTakeoutsGetOutput
# mapDashboardInstanceProviderOauthTakeoutsListOutput
# mapDashboardInstanceProviderOauthTakeoutsListQuery
# mapDashboardInstanceSecretsGetOutput
# mapDashboardInstanceSecretsListOutput
# mapDashboardInstanceSecretsListQuery
# mapDashboardInstanceServerConfigVaultsCreateBody
# mapDashboardInstanceServerConfigVaultsCreateOutput
# mapDashboardInstanceServerConfigVaultsGetOutput
# mapDashboardInstanceServerConfigVaultsListOutput
# mapDashboardInstanceServerConfigVaultsListQuery
# mapDashboardInstanceServerConfigVaultsUpdateBody
# mapDashboardInstanceServerConfigVaultsUpdateOutput
# mapDashboardInstanceServerRunErrorGroupsGetOutput
# mapDashboardInstanceServerRunErrorGroupsListOutput
# mapDashboardInstanceServerRunErrorGroupsListQuery
# mapDashboardInstanceServerRunErrorsGetOutput
# mapDashboardInstanceServerRunErrorsListOutput
# mapDashboardInstanceServerRunErrorsListQuery
# mapDashboardInstanceServerRunsGetOutput
# mapDashboardInstanceServerRunsListOutput
# mapDashboardInstanceServerRunsListQuery
# mapDashboardInstanceServersCapabilitiesListOutput
# mapDashboardInstanceServersCapabilitiesListQuery
# mapDashboardInstanceServersDeploymentsCreateBody
# mapDashboardInstanceServersDeploymentsCreateOutput
# mapDashboardInstanceServersDeploymentsDeleteOutput
# mapDashboardInstanceServersDeploymentsGetOutput
# mapDashboardInstanceServersDeploymentsListOutput
# mapDashboardInstanceServersDeploymentsListQuery
# mapDashboardInstanceServersDeploymentsUpdateBody
# mapDashboardInstanceServersDeploymentsUpdateOutput
# mapDashboardInstanceServersGetOutput
# mapDashboardInstanceServersImplementationsCreateBody
# mapDashboardInstanceServersImplementationsCreateOutput
# mapDashboardInstanceServersImplementationsDeleteOutput
# mapDashboardInstanceServersImplementationsGetOutput
# mapDashboardInstanceServersImplementationsListOutput
# mapDashboardInstanceServersImplementationsListQuery
# mapDashboardInstanceServersImplementationsUpdateBody
# mapDashboardInstanceServersImplementationsUpdateOutput
# mapDashboardInstanceServersVariantsGetOutput
# mapDashboardInstanceServersVariantsListOutput
# mapDashboardInstanceServersVariantsListQuery
# mapDashboardInstanceServersVersionsGetOutput
# mapDashboardInstanceServersVersionsListOutput
# mapDashboardInstanceServersVersionsListQuery
# mapDashboardInstanceSessionsConnectionsGetOutput
# mapDashboardInstanceSessionsConnectionsListOutput
# mapDashboardInstanceSessionsConnectionsListQuery
# mapDashboardInstanceSessionsCreateBody
# mapDashboardInstanceSessionsCreateOutput
# mapDashboardInstanceSessionsDeleteOutput
# mapDashboardInstanceSessionsEventsGetOutput
# mapDashboardInstanceSessionsEventsListOutput
# mapDashboardInstanceSessionsEventsListQuery
# mapDashboardInstanceSessionsGetOutput
# mapDashboardInstanceSessionsListOutput
# mapDashboardInstanceSessionsListQuery
# mapDashboardInstanceSessionsMessagesGetOutput
# mapDashboardInstanceSessionsMessagesListOutput
# mapDashboardInstanceSessionsMessagesListQuery
# mapDashboardInstanceSessionsServerSessionsGetOutput
# mapDashboardInstanceSessionsServerSessionsListOutput
# mapDashboardInstanceSessionsServerSessionsListQuery
# mapDashboardOrganizationsCreateBody
# mapDashboardOrganizationsCreateOutput
# mapDashboardOrganizationsDeleteOutput
# mapDashboardOrganizationsGetOutput
# mapDashboardOrganizationsInstancesCreateBody
# mapDashboardOrganizationsInstancesCreateOutput
# mapDashboardOrganizationsInstancesDeleteOutput
# mapDashboardOrganizationsInstancesGetOutput
# mapDashboardOrganizationsInstancesListOutput
# mapDashboardOrganizationsInstancesListQuery
# mapDashboardOrganizationsInstancesUpdateBody
# mapDashboardOrganizationsInstancesUpdateOutput
# mapDashboardOrganizationsInvitesCreateBody
# mapDashboardOrganizationsInvitesCreateOutput
# mapDashboardOrganizationsInvitesDeleteOutput
# mapDashboardOrganizationsInvitesEnsureLinkOutput
# mapDashboardOrganizationsInvitesGetOutput
# mapDashboardOrganizationsInvitesListOutput
# mapDashboardOrganizationsInvitesListQuery
# mapDashboardOrganizationsInvitesUpdateBody
# mapDashboardOrganizationsInvitesUpdateOutput
# mapDashboardOrganizationsJoinAcceptBody
# mapDashboardOrganizationsJoinAcceptOutput
# mapDashboardOrganizationsJoinGetOutput
# mapDashboardOrganizationsJoinGetQuery
# mapDashboardOrganizationsJoinRejectBody
# mapDashboardOrganizationsJoinRejectOutput
# mapDashboardOrganizationsListOutput
# mapDashboardOrganizationsListQuery
# mapDashboardOrganizationsMembersDeleteOutput
# mapDashboardOrganizationsMembersGetOutput
# mapDashboardOrganizationsMembersListOutput
# mapDashboardOrganizationsMembersListQuery
# mapDashboardOrganizationsMembersUpdateBody
# mapDashboardOrganizationsMembersUpdateOutput
# mapDashboardOrganizationsProjectsCreateBody
# mapDashboardOrganizationsProjectsCreateOutput
# mapDashboardOrganizationsProjectsDeleteOutput
# mapDashboardOrganizationsProjectsGetOutput
# mapDashboardOrganizationsProjectsListOutput
# mapDashboardOrganizationsProjectsListQuery
# mapDashboardOrganizationsProjectsUpdateBody
# mapDashboardOrganizationsProjectsUpdateOutput
# mapDashboardOrganizationsTeamsCreateBody
# mapDashboardOrganizationsTeamsCreateOutput
# mapDashboardOrganizationsTeamsGetOutput
# mapDashboardOrganizationsTeamsListOutput
# mapDashboardOrganizationsTeamsListQuery
# mapDashboardOrganizationsTeamsMembersCreateBody
# mapDashboardOrganizationsTeamsMembersCreateOutput
# mapDashboardOrganizationsTeamsMembersDeleteOutput
# mapDashboardOrganizationsTeamsPermissionsOutput
# mapDashboardOrganizationsTeamsProjectsRemoveOutput
# mapDashboardOrganizationsTeamsProjectsSetBody
# mapDashboardOrganizationsTeamsProjectsSetOutput
# mapDashboardOrganizationsTeamsRolesCreateBody
# mapDashboardOrganizationsTeamsRolesCreateOutput
# mapDashboardOrganizationsTeamsRolesGetOutput
# mapDashboardOrganizationsTeamsRolesListOutput
# mapDashboardOrganizationsTeamsRolesListQuery
# mapDashboardOrganizationsTeamsRolesUpdateBody
# mapDashboardOrganizationsTeamsRolesUpdateOutput
# mapDashboardOrganizationsTeamsUpdateBody
# mapDashboardOrganizationsTeamsUpdateOutput
# mapDashboardOrganizationsUpdateBody
# mapDashboardOrganizationsUpdateOutput
# mapDashboardScmAccountsPreviewOutput
# mapDashboardScmAccountsPreviewQuery
# mapDashboardScmInstallationsCreateBody
# mapDashboardScmInstallationsCreateOutput
# mapDashboardScmInstallationsGetOutput
# mapDashboardScmInstallationsListOutput
# mapDashboardScmInstallationsListQuery
# mapDashboardScmReposCreateBody
# mapDashboardScmReposCreateOutput
# mapDashboardScmReposPreviewOutput
# mapDashboardScmReposPreviewQuery
# mapDashboardUsageTimelineOutput
# mapDashboardUsageTimelineQuery
# mapManagementOrganizationGetOutput
# mapManagementOrganizationUpdateBody
# mapManagementOrganizationUpdateOutput
# mapManagementUserDeleteBody
# mapManagementUserDeleteOutput
# mapManagementUserGetOutput
# mapManagementUserUpdateBody
# mapManagementUserUpdateOutput
# mapOrganizationsProfileGetOutput
# mapOrganizationsProfileUpdateBody
# mapOrganizationsProfileUpdateOutput
# mapProviderOauthConnectionTemplateEvaluateBody
# mapProviderOauthConnectionTemplateEvaluateOutput
# mapProviderOauthConnectionTemplateGetOutput
# mapProviderOauthConnectionTemplateListOutput
# mapProviderOauthConnectionTemplateListQuery
# mapProviderOauthDiscoverBody
# mapProviderOauthDiscoverOutput
# mapServersListingsCategoriesGetOutput
# mapServersListingsCategoriesListOutput
# mapServersListingsCategoriesListQuery
# mapServersListingsCollectionsGetOutput
# mapServersListingsCollectionsListOutput
# mapServersListingsCollectionsListQuery
# mapServersListingsGetOutput
# mapServersListingsGetQuery
# mapServersListingsListOutput
# mapServersListingsListQuery

# All types and mappers are available via:
# from .resources import *
# from .endpoints import *



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardInstanceSessionsGetOutputClientSecret:
  object: str
  type: str
  id: str
  secret: str
  expires_at: datetime


@dataclass
class DashboardInstanceSessionsGetOutputServerDeploymentsServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
  sse: str
  streamable_http: str


@dataclass
class DashboardInstanceSessionsGetOutputServerDeployments:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: DashboardInstanceSessionsGetOutputServerDeploymentsServer
  connection_urls: DashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls
  name: Optional[str] = None
  oauth_session_id: Optional[str] = None
  description: Optional[str] = None


@dataclass
class DashboardInstanceSessionsGetOutputUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class DashboardInstanceSessionsGetOutput:
  object: str
  id: str
  status: str
  connection_status: str
  client_secret: DashboardInstanceSessionsGetOutputClientSecret
  server_deployments: List[DashboardInstanceSessionsGetOutputServerDeployments]
  usage: DashboardInstanceSessionsGetOutputUsage
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime


class mapDashboardInstanceSessionsGetOutputClientSecret:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceSessionsGetOutputClientSecret:
    return DashboardInstanceSessionsGetOutputClientSecret(
      object=data.get("object"),
      type=data.get("type"),
      id=data.get("id"),
      secret=data.get("secret"),
      expires_at=parse_iso_datetime(data.get("expires_at"))
      if data.get("expires_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceSessionsGetOutputClientSecret, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsGetOutputServerDeploymentsServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsGetOutputServerDeploymentsServer:
    return DashboardInstanceSessionsGetOutputServerDeploymentsServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsGetOutputServerDeploymentsServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls:
    return DashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls(
      sse=data.get("sse"), streamable_http=data.get("streamable_http")
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsGetOutputServerDeployments:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceSessionsGetOutputServerDeployments:
    return DashboardInstanceSessionsGetOutputServerDeployments(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      oauth_session_id=data.get("oauth_session_id"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapDashboardInstanceSessionsGetOutputServerDeploymentsServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      connection_urls=mapDashboardInstanceSessionsGetOutputServerDeploymentsConnectionUrls.from_dict(
        data.get("connection_urls")
      )
      if data.get("connection_urls")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceSessionsGetOutputServerDeployments, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsGetOutputUsage:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceSessionsGetOutputUsage:
    return DashboardInstanceSessionsGetOutputUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceSessionsGetOutputUsage, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceSessionsGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceSessionsGetOutput:
    return DashboardInstanceSessionsGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      connection_status=data.get("connection_status"),
      client_secret=mapDashboardInstanceSessionsGetOutputClientSecret.from_dict(
        data.get("client_secret")
      )
      if data.get("client_secret")
      else None,
      server_deployments=[
        mapDashboardInstanceSessionsGetOutputServerDeployments.from_dict(item)
        for item in data.get("server_deployments", [])
        if item
      ],
      usage=mapDashboardInstanceSessionsGetOutputUsage.from_dict(data.get("usage"))
      if data.get("usage")
      else None,
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceSessionsGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceCallbacksEventsGetOutput:
    object: str
    id: str
    type: str
    source_id: str
    trigger_key: str
    input: Dict[str, Any]
    output: Dict[str, Any]
    delivery_status: str
    callback_id: str
    created_at: datetime
    callback_instance_id: Optional[str] = None


class mapDashboardInstanceCallbacksEventsGetOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCallbacksEventsGetOutput:
        return DashboardInstanceCallbacksEventsGetOutput(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        source_id=data.get('source_id'),
        trigger_key=data.get('trigger_key'),
        input=data.get('input'),
        output=data.get('output'),
        delivery_status=data.get('delivery_status'),
        callback_id=data.get('callback_id'),
        callback_instance_id=data.get('callback_instance_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCallbacksEventsGetOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer:
  id: str
  status: str
  name: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class DashboardInstanceMagicMcpSessionsListOutputItemsUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class DashboardInstanceMagicMcpSessionsListOutputItems:
  object: str
  id: str
  session_id: str
  connection_status: str
  magic_mcp_server: DashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer
  usage: DashboardInstanceMagicMcpSessionsListOutputItemsUsage
  created_at: datetime
  updated_at: datetime


@dataclass
class DashboardInstanceMagicMcpSessionsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class DashboardInstanceMagicMcpSessionsListOutput:
  items: List[DashboardInstanceMagicMcpSessionsListOutputItems]
  pagination: DashboardInstanceMagicMcpSessionsListOutputPagination


class mapDashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer:
    return DashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer(
      id=data.get("id"),
      status=data.get("status"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceMagicMcpSessionsListOutputItemsUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceMagicMcpSessionsListOutputItemsUsage:
    return DashboardInstanceMagicMcpSessionsListOutputItemsUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceMagicMcpSessionsListOutputItemsUsage, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceMagicMcpSessionsListOutputItems:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceMagicMcpSessionsListOutputItems:
    return DashboardInstanceMagicMcpSessionsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      session_id=data.get("session_id"),
      connection_status=data.get("connection_status"),
      magic_mcp_server=mapDashboardInstanceMagicMcpSessionsListOutputItemsMagicMcpServer.from_dict(
        data.get("magic_mcp_server")
      )
      if data.get("magic_mcp_server")
      else None,
      usage=mapDashboardInstanceMagicMcpSessionsListOutputItemsUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceMagicMcpSessionsListOutputItems, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceMagicMcpSessionsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceMagicMcpSessionsListOutputPagination:
    return DashboardInstanceMagicMcpSessionsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceMagicMcpSessionsListOutputPagination, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceMagicMcpSessionsListOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpSessionsListOutput:
    return DashboardInstanceMagicMcpSessionsListOutput(
      items=[
        mapDashboardInstanceMagicMcpSessionsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapDashboardInstanceMagicMcpSessionsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceMagicMcpSessionsListOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardInstanceMagicMcpSessionsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None
  magic_mcp_server_id: Optional[Union[str, List[str]]] = None


class mapDashboardInstanceMagicMcpSessionsListQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpSessionsListQuery:
    return DashboardInstanceMagicMcpSessionsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
      magic_mcp_server_id=data.get("magic_mcp_server_id"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceMagicMcpSessionsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from typing import Any, Dict, List, Optional, Union
from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardInstanceScmInstallationListOutput, DashboardInstanceScmInstallationListOutput, mapDashboardInstanceScmInstallationListQuery, DashboardInstanceScmInstallationListQuery, mapDashboardInstanceScmInstallationCreateOutput, DashboardInstanceScmInstallationCreateOutput, mapDashboardInstanceScmInstallationCreateBody, DashboardInstanceScmInstallationCreateBody

class MetorialScmInstallationEndpoint(BaseMetorialEndpoint):
    """Manage source control management installations (e.g. GitHub App installations)."""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None) -> DashboardInstanceScmInstallationListOutput:
        """
    List SCM installations
    Returns a paginated list of SCM installations.

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: DashboardInstanceScmInstallationListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order

        request = MetorialRequest(
            path=['scm', 'installations'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardInstanceScmInstallationListOutput.from_dict)

    def create(self, *, provider: Optional[str] = None, redirect_url: Optional[str] = None) -> DashboardInstanceScmInstallationCreateOutput:
        """
    Create SCM installation
    Initiates an SCM installation setup (e.g. GitHub App authorization).

    :param provider: Optional[str] (optional)
    :param redirect_url: Optional[str] (optional)
    :return: DashboardInstanceScmInstallationCreateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        if provider is not None:
            body_dict["provider"] = provider
        if redirect_url is not None:
            body_dict["redirect_url"] = redirect_url

        request = MetorialRequest(
            path=['scm', 'installations'],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardInstanceScmInstallationCreateOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputConfigSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputConfig:
    object: str
    schema: ManagementInstanceCustomProvidersVersionsGetOutputConfigSchema
    transformer: str
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit:
    object: str
    id: str
    type: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository:
    object: str
    id: str
    provider: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink:
    object: str
    is_linked: str
    repository: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository
    path: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket:
    object: str
    id: str
    is_immutable: bool
    is_read_only: bool
    created_at: datetime
    scm_repo_link: Optional[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    created_at: datetime
    organization_actor_id: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor:
    object: str
    id: str
    external_id: Optional[str] = None
    name: Optional[str] = None
    email: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit:
    object: str
    id: str
    sha: str
    branch: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository:
    object: str
    id: str
    provider: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush:
    object: str
    id: str
    actor: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor
    commit: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit
    repository: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository
    created_at: datetime
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputDeployment:
    object: str
    id: str
    status: str
    trigger: str
    custom_provider_id: str
    actor: ManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor
    created_at: datetime
    updated_at: datetime
    provider_id: Optional[str] = None
    custom_provider_version_id: Optional[str] = None
    commit: Optional[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit] = None
    immutable_bucket: Optional[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket] = None
    scm_push: Optional[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment:
    object: str
    id: str
    custom_provider_id: str
    instance_id: str
    created_at: datetime
    updated_at: datetime
    provider_id: Optional[str] = None
    current_provider_version_id: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputEnvironments:
    object: str
    id: str
    is_current_version_for_environment: bool
    environment: ManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputActor:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    created_at: datetime
    organization_actor_id: Optional[str] = None
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputContainerImage:
    container_registry: str
    container_image_tag: str
    container_image: str
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer:
    url: str
    transport: str
@dataclass
class ManagementInstanceCustomProvidersVersionsGetOutput:
    object: str
    id: str
    status: str
    index: float
    identifier: str
    deployment: ManagementInstanceCustomProvidersVersionsGetOutputDeployment
    environments: List[ManagementInstanceCustomProvidersVersionsGetOutputEnvironments]
    custom_provider_id: str
    actor: ManagementInstanceCustomProvidersVersionsGetOutputActor
    created_at: datetime
    updated_at: datetime
    config: Optional[ManagementInstanceCustomProvidersVersionsGetOutputConfig] = None
    provider_id: Optional[str] = None
    container_image: Optional[ManagementInstanceCustomProvidersVersionsGetOutputContainerImage] = None
    remote_mcp_server: Optional[ManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer] = None


class mapManagementInstanceCustomProvidersVersionsGetOutputConfigSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputConfigSchema:
        return ManagementInstanceCustomProvidersVersionsGetOutputConfigSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputConfigSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputConfig:
        return ManagementInstanceCustomProvidersVersionsGetOutputConfig(
        object=data.get('object'),
        schema=mapManagementInstanceCustomProvidersVersionsGetOutputConfigSchema.from_dict(data.get('schema')) if data.get('schema') else None,
        transformer=data.get('transformer')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        message=data.get('message'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider(
        object=data.get('object'),
        type=data.get('type'),
        id=data.get('id'),
        name=data.get('name'),
        owner=data.get('owner')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository(
        object=data.get('object'),
        id=data.get('id'),
        provider=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        url=data.get('url'),
        is_private=data.get('is_private'),
        default_branch=data.get('default_branch'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink(
        object=data.get('object'),
        is_linked=data.get('is_linked'),
        path=data.get('path'),
        repository=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLinkRepository.from_dict(data.get('repository')) if data.get('repository') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket(
        object=data.get('object'),
        id=data.get('id'),
        is_immutable=data.get('is_immutable'),
        is_read_only=data.get('is_read_only'),
        scm_repo_link=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucketScmRepoLink.from_dict(data.get('scm_repo_link')) if data.get('scm_repo_link') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        organization_actor_id=data.get('organization_actor_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor(
        object=data.get('object'),
        id=data.get('id'),
        external_id=data.get('external_id'),
        name=data.get('name'),
        email=data.get('email')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit(
        object=data.get('object'),
        id=data.get('id'),
        sha=data.get('sha'),
        branch=data.get('branch'),
        message=data.get('message'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider(
        object=data.get('object'),
        type=data.get('type'),
        id=data.get('id'),
        name=data.get('name'),
        owner=data.get('owner')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository(
        object=data.get('object'),
        id=data.get('id'),
        provider=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepositoryProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        url=data.get('url'),
        is_private=data.get('is_private'),
        default_branch=data.get('default_branch'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush(
        object=data.get('object'),
        id=data.get('id'),
        actor=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushActor.from_dict(data.get('actor')) if data.get('actor') else None,
        commit=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushCommit.from_dict(data.get('commit')) if data.get('commit') else None,
        repository=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPushRepository.from_dict(data.get('repository')) if data.get('repository') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputDeployment:
        return ManagementInstanceCustomProvidersVersionsGetOutputDeployment(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        trigger=data.get('trigger'),
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        custom_provider_version_id=data.get('custom_provider_version_id'),
        commit=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentCommit.from_dict(data.get('commit')) if data.get('commit') else None,
        immutable_bucket=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentImmutableBucket.from_dict(data.get('immutable_bucket')) if data.get('immutable_bucket') else None,
        actor=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentActor.from_dict(data.get('actor')) if data.get('actor') else None,
        scm_push=mapManagementInstanceCustomProvidersVersionsGetOutputDeploymentScmPush.from_dict(data.get('scm_push')) if data.get('scm_push') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment:
        return ManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment(
        object=data.get('object'),
        id=data.get('id'),
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        current_provider_version_id=data.get('current_provider_version_id'),
        instance_id=data.get('instance_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputEnvironments:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputEnvironments:
        return ManagementInstanceCustomProvidersVersionsGetOutputEnvironments(
        object=data.get('object'),
        id=data.get('id'),
        is_current_version_for_environment=data.get('is_current_version_for_environment'),
        environment=mapManagementInstanceCustomProvidersVersionsGetOutputEnvironmentsEnvironment.from_dict(data.get('environment')) if data.get('environment') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputEnvironments, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputActor:
        return ManagementInstanceCustomProvidersVersionsGetOutputActor(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        organization_actor_id=data.get('organization_actor_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputContainerImage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputContainerImage:
        return ManagementInstanceCustomProvidersVersionsGetOutputContainerImage(
        container_registry=data.get('container_registry'),
        container_image_tag=data.get('container_image_tag'),
        container_image=data.get('container_image')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputContainerImage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer:
        return ManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer(
        url=data.get('url'),
        transport=data.get('transport')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceCustomProvidersVersionsGetOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceCustomProvidersVersionsGetOutput:
        return ManagementInstanceCustomProvidersVersionsGetOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        config=mapManagementInstanceCustomProvidersVersionsGetOutputConfig.from_dict(data.get('config')) if data.get('config') else None,
        index=data.get('index'),
        identifier=data.get('identifier'),
        deployment=mapManagementInstanceCustomProvidersVersionsGetOutputDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        environments=[mapManagementInstanceCustomProvidersVersionsGetOutputEnvironments.from_dict(item) for item in data.get('environments', []) if item],
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        actor=mapManagementInstanceCustomProvidersVersionsGetOutputActor.from_dict(data.get('actor')) if data.get('actor') else None,
        container_image=mapManagementInstanceCustomProvidersVersionsGetOutputContainerImage.from_dict(data.get('container_image')) if data.get('container_image') else None,
        remote_mcp_server=mapManagementInstanceCustomProvidersVersionsGetOutputRemoteMcpServer.from_dict(data.get('remote_mcp_server')) if data.get('remote_mcp_server') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceCustomProvidersVersionsGetOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputCommit:
    object: str
    id: str
    type: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLinkRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLinkRepository:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLinkRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLink:
    object: str
    is_linked: str
    repository: DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLinkRepository
    path: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucket:
    object: str
    id: str
    is_immutable: bool
    is_read_only: bool
    created_at: datetime
    scm_repo_link: Optional[DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucketScmRepoLink] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputActor:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    created_at: datetime
    organization_actor_id: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushActor:
    object: str
    id: str
    external_id: Optional[str] = None
    name: Optional[str] = None
    email: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushCommit:
    object: str
    id: str
    sha: str
    branch: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushRepository:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutputScmPush:
    object: str
    id: str
    actor: DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushActor
    commit: DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushCommit
    repository: DashboardInstanceCustomProvidersDeploymentsGetOutputScmPushRepository
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersDeploymentsGetOutput:
    object: str
    id: str
    status: str
    trigger: str
    custom_provider_id: str
    actor: DashboardInstanceCustomProvidersDeploymentsGetOutputActor
    created_at: datetime
    updated_at: datetime
    provider_id: Optional[str] = None
    custom_provider_version_id: Optional[str] = None
    commit: Optional[DashboardInstanceCustomProvidersDeploymentsGetOutputCommit] = None
    immutable_bucket: Optional[DashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucket] = None
    scm_push: Optional[DashboardInstanceCustomProvidersDeploymentsGetOutputScmPush] = None


class mapDashboardInstanceCustomProvidersDeploymentsGetOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersDeploymentsGetOutput:
        return DashboardInstanceCustomProvidersDeploymentsGetOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        trigger=data.get('trigger'),
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        custom_provider_version_id=data.get('custom_provider_version_id'),
        commit=mapDashboardInstanceCustomProvidersDeploymentsGetOutputCommit.from_dict(data.get('commit')) if data.get('commit') else None,
        immutable_bucket=mapDashboardInstanceCustomProvidersDeploymentsGetOutputImmutableBucket.from_dict(data.get('immutable_bucket')) if data.get('immutable_bucket') else None,
        actor=mapDashboardInstanceCustomProvidersDeploymentsGetOutputActor.from_dict(data.get('actor')) if data.get('actor') else None,
        scm_push=mapDashboardInstanceCustomProvidersDeploymentsGetOutputScmPush.from_dict(data.get('scm_push')) if data.get('scm_push') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersDeploymentsGetOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

